--[[	PhantomPlates Options Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local Name,AddOn=...;
local Title=select(2,GetAddOnInfo(Name)):gsub("%s*v?[%d%.]+$","");
local Version=GetAddOnMetadata(Name,"Version");
local Author=GetAddOnMetadata(Name,"Author");

local OptionsPanel=CreateFrame("Frame");
AddOn.OptionsPanel=OptionsPanel;
OptionsPanel:Hide();

--------------------------
--[[	Options Tables	]]
--------------------------
local Defaults,Options={},nil;
AddOn.Options=Defaults;--	Modules can write to defaults while loading

--------------------------
--[[	Local Variables	]]
--------------------------
local OptionsCache={};

local OptionsLayoutSectionAnchors={
	{LEFT={"LEFT",8,0},RIGHT={"CENTER",-2,0}};
	{LEFT={"CENTER",2,0},RIGHT={"RIGHT",-8,0}};
};
OptionsLayoutSectionObjects={};
for i in ipairs(OptionsLayoutSectionAnchors) do OptionsLayoutSectionObjects[i]={}; end

local OptionsPanelControls=setmetatable({},{__index=function(t,k)
	local proxy={Key=k,OnValueChanged=AddOn.CreateDispatcher()};
	t[k]=proxy; return proxy;
end});

local OptionsPanelControlDataMap={};

----------------------------------
--[[	Options Panel Layout	]]
----------------------------------
OptionsPanel.name=Title;
do	local title=OptionsPanel:CreateFontString(nil,"OVERLAY","GameFontNormalLarge");
	title:SetPoint("TOP",0,-12);
	title:SetText(Title);

	local author=OptionsPanel:CreateFontString(nil,"OVERLAY","GameFontNormalSmall");
	author:SetPoint("TOP",title,"BOTTOM");
	author:SetTextColor(1,0.5,0.25);
	author:SetFormattedText(AddOn.Localization.Options_ByAuthor_Format,Author);

	local ver=OptionsPanel:CreateFontString(nil,"OVERLAY","GameFontNormalSmall");
	ver:SetPoint("TOPLEFT",title,"TOPRIGHT",4,0);
	ver:SetTextColor(0.5,0.5,0.5);
	ver:SetText("v"..Version);
end

InterfaceOptions_AddCategory(OptionsPanel);--	Panel Registration

------------------------------------------
--[[	Options Panel Event Handler	]]
------------------------------------------
OptionsPanel:RegisterEvent("ADDON_LOADED");
OptionsPanel:SetScript("OnEvent",function(self,event,...)
	if event=="ADDON_LOADED" and (...)==Name then
--		Load Options
		local rawoptions=PhantomPlates_Options;
		if not rawoptions then rawoptions={}; PhantomPlates_Options=rawoptions; end

--		Trim/Fill Options
--		for key,val in pairs(rawoptions) do if Defaults[key]==nil then rawoptions[key]=nil; end end
		for key,val in pairs(Defaults) do if rawoptions[key]==nil then rawoptions[key]=val; end end

--		Attach Metatable
		Options=setmetatable({},{
			__index=rawoptions;
			__newindex=function(t,k,v)
				if rawoptions[k]~=v then
					rawoptions[k]=v;
					local data=OptionsPanelControls[k];
					data.OnValueChanged(data.Control,k,v);
				end
			end;
		});
		AddOn.Options=Options;		

--		Send updates to modules
		for key,data in pairs(OptionsPanelControls) do
			data.OnValueChanged(data.Control,key,rawoptions[key]);
		end

--		Don't need this event anymore
		self:UnregisterEvent(event);
	end
end);

----------------------------------
--[[	Options Panel Callbacks	]]
----------------------------------
function OptionsPanel.okay()
	table.wipe(OptionsCache);--	Options already set by preview, just cleanup
end

function OptionsPanel.cancel()
	for key,val in pairs(OptionsCache) do
		if Options[key]~=val then Options[key]=val; end--	Restore previous setting
	end
	table.wipe(OptionsCache);--	Cleanup
end

function OptionsPanel.refresh()
	table.wipe(OptionsCache);--	Sanity check

	local errhnd=geterrorhandler();
	for key,data in pairs(OptionsPanelControls) do
		if data.Control then
			if data.Refresh then data.Refresh(data.Control); end
			if data.SetValue then data.SetValue(data.Control,Options[key]); end
		end
	end
end

function OptionsPanel.default()
	local errhnd=geterrorhandler();
	for key,data in pairs(OptionsPanelControls) do
		local val=Defaults[key]; if Options[key]~=val then
			Options[key]=val;--	Restore default
			if data.SetValue then data.SetValue(data.Control,val); end
		end
	end
end

--------------------------
--[[	Options API	]]
--------------------------
--	Options Panel Layout
function AddOn.OptionsPanel:GetSectionHeight(section)
	local height=0; for _,obj in ipairs(OptionsLayoutSectionObjects[section]) do
		height=height+obj:GetHeight();
	end
	return height;
end

function AddOn.OptionsPanel:GetPreferredSection()
	local lowsection,lowheight;
	for i=1,#OptionsLayoutSectionObjects do
		local height=self:GetSectionHeight(i);
		if not lowheight or height<lowheight then
			lowsection,lowheight=i,height;
		end
	end
	return lowsection;
end

function AddOn.OptionsPanel:GetLastSectionObject(section)
	section=OptionsLayoutSectionObjects[section];
	return section[#section];
end

function AddOn.OptionsPanel:AddSectionObject(obj,section)
	table.insert(OptionsLayoutSectionObjects[section],obj);
end

function AddOn.OptionsPanel:CreateSubPanel(section)
	section=section or self:GetPreferredSection();

	local subpanel=CreateFrame("Frame",nil,self,DoesTemplateExist("BackdropTemplate") and "BackdropTemplate" or nil);
	subpanel:SetBackdrop(BACKDROP_TOOLTIP_0_16_5555 or {
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border";
		edgeSize=16;
		tileEdge=true;
		insets={left=5,right=5,top=5,bottom=5};
	});
	subpanel:SetBackdropBorderColor(LIGHTGRAY_FONT_COLOR:GetRGB());

--	Top Anchor
	local prevobj=self:GetLastSectionObject(section);
	if prevobj then subpanel:SetPoint("TOP",prevobj,"BOTTOM",0,-4);
	else subpanel:SetPoint("TOP",0,-48); end

--	Side Anchors
	for point,anchor in pairs(OptionsLayoutSectionAnchors[section]) do
		if point==anchor[1] then subpanel:SetPoint(unpack(anchor));
		else subpanel:SetPoint(point,self,unpack(anchor)); end
	end

	self:AddSectionObject(subpanel,section);
	return subpanel;
end

--	Options Panel Controls
local function OptionsControlObject_OnUpdateScript(self)
	local data=OptionsPanelControlDataMap[self];
	if data.GetValue then
		local key,val=data.Key,data.GetValue(self);
		if Options[key]~=val then Options[key]=val; end
	end
end

function AddOn.OptionsPanel:RegisterControl(key,control,refreshfunc,setfunc,getfunc,updatescript)
	local data=OptionsPanelControls[key];--	Metatable creates basic proxy as needed
	OptionsPanelControlDataMap[control]=data;

	data.Control=control;
	data.Refresh=refreshfunc;
	data.SetValue=setfunc;
	data.GetValue=getfunc;

	if updatescript then control:SetScript(updatescript,OptionsControlObject_OnUpdateScript); end
	return proxy;
end

function AddOn.OptionsPanel:GetControl(key) return OptionsPanelControls[key].Control; end
function AddOn.OptionsPanel:HookOption(key,func) OptionsPanelControls[key].OnValueChanged:AddCallback(func); end
